using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace Maze {
	public class Maze {
		public int Seed { get; private set; }

		private int roomNumber;
		public int RoomNumber {
			get {
				return roomNumber;
			}
			set {
				roomNumber = value;
				GenerateRoom();
			}
		}

		public int Height { get; private set; }
		public int Width { get; private set; }
		public int NumberOfDirections { get; private set; }

		private Dictionary<int, int[,]> roomCache;

		public Maze(int width, int height, int numberOfDirections) : this(width, height, numberOfDirections, Environment.TickCount) { }

		public Maze(int width, int height, int numberOfDirections, int seed) {
			roomCache = new Dictionary<int, int[,]>();
			Height = height;
			Width = width;
			NumberOfDirections = numberOfDirections;
			Seed = seed;
			RoomNumber = seed;
		}

		public int[,] GenerateRoom(int? roomNumber = null) {
			//if (roomNumber != null) { RoomNumber = roomNumber.Value; }

			if (roomCache.ContainsKey(RoomNumber)) {
				return roomCache[RoomNumber];
			}

			int[,] ret = new int[Height, Width];
			Array directions = Enum.GetValues(typeof(Direction));
			Random rand = new Random(RoomNumber);
			for (int i = 0; i < Height; i++) {
				for (int j = 0; j < Width; j++) {
					ret[i, j] = rand.Next(NumberOfDirections);
				}
			}
			
			roomCache[RoomNumber] = ret;
			return ret;
		}
	}
}
